// (C) 1997, 1998 by Paolo Gatti
//
// $Id: GridTest.cpp,v 1.7 1998/11/01 21:44:29 paolo Exp $
// $Log: GridTest.cpp,v $
// Revision 1.7  1998/11/01 21:44:29  paolo
// R4 compatibility
//
// Revision 1.6  1998/01/05 21:00:54  paolo
// First public release
//
// Revision 1.5  1998/01/03 22:19:34  paolo
// Now uses Replicant technology
//
// Revision 1.2  1997/12/22 21:12:24  paolo
// Changed window size
//
// Revision 1.1  1997/12/14 20:24:09  paolo
// Initial revision

#include "GridView.h"

static const char *const signature = "application/x-vnd.pg-gridtest";

class GenericApp : public BApplication
{
	public:
		GenericApp();
		~GenericApp();
};

class GenericWindow : public BWindow
{
	public:
		GenericWindow();
		~GenericWindow();
};

// This class's symbol is exported to allow MyGridView Replicants
class __declspec(dllexport) MyGridView : public GridView
{
	public:
		MyGridView(	BRect frame,
					const char *name,
					uint32 resizingMode,
					uint32 flags,
					float bordersize,
					float minseparation);
		MyGridView(BMessage *archive);
		virtual ~MyGridView();
		static BArchivable *Instantiate(BMessage *archive);
		virtual status_t Archive(BMessage *archive, bool deep) const;
		virtual void DrawEmptyChild(BRect frame, BRect updateRect);
		virtual void ClickOnChild(BPoint where, int32 x, int32 y);
		virtual void ChildDetached(BView *child);
		virtual void NewChild(int32 id);
};

// Constructors and destructors don't do much
MyGridView::MyGridView(BRect frame, const char *name,
	uint32 resizingMode = B_FOLLOW_NONE, uint32 flags = NULL,
	float bordersize = 2, float minseparation = 0) :
GridView(frame, name, resizingMode, flags, bordersize, minseparation)
{
	;
}

MyGridView::MyGridView(BMessage *archive) : GridView(archive)
{
	;
}

MyGridView::~MyGridView()
{
	;
}

// Replicants
BArchivable *MyGridView::Instantiate(BMessage *archive)
{
	if (validate_instantiation(archive, "MyGridView"))
		return new MyGridView(archive); 
	return NULL; 
}

status_t MyGridView::Archive(BMessage *archive, bool deep = true) const
{
	GridView::Archive(archive, deep);
	archive->AddString("class", "MyGridView");
	archive->AddString("add_on", signature);

	return B_OK;
}

// How should empty children be drawn?
void MyGridView::DrawEmptyChild(BRect frame, BRect updateRect)
{
	SetHighColor(
			int32(frame.left)%256,
			int32(frame.top)%256,
			int32(frame.bottom)%256
	);
	FillRect(updateRect);
}

// What happens if the user clicks on an empty rectangle?
void MyGridView::ClickOnChild(BPoint where, int32 x, int32 y)
{
	AddChild(new BButton(BRect(0,0,100,100), "button", "Test",
						 new BMessage('abcd')), x, y);
}

// What if a child view is resized to oblivion?
void MyGridView::ChildDetached(BView *view)
{
	printf("View destroyed\n");
	delete view;
}

// Called when a new rectangle is added
void MyGridView::NewChild(int32 id)
{
	printf("Child %ld added\n", id);
}

// Launch the demo
int main()
{
	new GenericApp;
	be_app->Run();
	delete be_app;
}

GenericApp::GenericApp() : BApplication(signature)
{
	new GenericWindow;
}

GenericApp::~GenericApp()
{ }

GenericWindow::~GenericWindow()
{
	be_app->PostMessage(B_QUIT_REQUESTED);	
}

// Set up a demo window, showing usage of some GridView features
GenericWindow::GenericWindow()
: BWindow(BRect(50,50,450,450),"Grid test window",B_TITLED_WINDOW,
			B_NOT_RESIZABLE|B_NOT_ZOOMABLE)
{
	MyGridView *g, *w;
	BDragger *d;
	BView *v;
	BRect bounds;
	BArchivable *unarchived;
	BMessage *msg = new BMessage(B_ARCHIVED_OBJECT);

// Create main view
	v = new BView(Bounds(), "Grid test view", B_FOLLOW_NONE, B_WILL_DRAW);
	v->SetViewColor(192,192,192);

// Test GridView
	bounds = Bounds();
	bounds.bottom -= 8;
	g = new MyGridView(bounds, "GridView", B_FOLLOW_NONE);

// Add and resize a few children:
// Drag the left border (0, 1) to the view's center (vertical split)
	g->DragSeparator(0, 1, 200, 100, B_PRIMARY_MOUSE_BUTTON);
// Drag the upper border of the right column (3, 0) down
	g->DragSeparator(3, 0, 200, 100, B_PRIMARY_MOUSE_BUTTON);
// Drag its lower border (3, 4) up
	g->DragSeparator(3, 4, 200, 250, B_PRIMARY_MOUSE_BUTTON);
// Resize the enture second column - make it a little narrower.
// Usage of SECONDARY_MOUSE_BUTTON resizes the entire column instead of
// moving only the central part of the separator.
	g->DragSeparator(2, 3, 250, 250, B_SECONDARY_MOUSE_BUTTON);

// Archive the GridView and restore it from the archive
	g->Archive(msg);
	unarchived = instantiate_object(msg);
	if (unarchived)
	{
		w = cast_as(unarchived, MyGridView);
		if (w)
		{
			delete g;
			delete msg; 
			g = w;
		}
	}

// Add the GridView to the main view
	v->AddChild(g);

// Make it replicatable
	bounds.top = bounds.bottom + 1;
	bounds.bottom += 8;
	bounds.left = bounds.right - 7;
	d = new BDragger(bounds, g);
	v->AddChild(d);
	
// Add it all to the window
	AddChild(v);
	Show();
}

